#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#       meshViewerDialog.py
#
#   コマンドの内容
#
#   show            指定したparts（patch, zone, set, stl）を表示
#   showActor        例: show patch:inW cellZone:block faceSet:faceF stl:outW
#
#   addShow         指定したparts（patch, zone, set, stl）を追加表示
#   addShowActor    例: addShow patch:inW cellZone:block faceSet:faceF
#
#   load            指定したvtkDataを読み込む
#   reload          例: load [all] [patchesZones] [sets] [stls]
#                       load sets stls     #setsとstlを再読込する。
#
#   case            caseDirを指定する。実行後は「load all」で再読込する
#   caseDir         例: case ~/test/cavity
#
#   region          regionを指定する。実行後は「load all」で再読込する。
#                   例: region .
#
#   time            timeFolderを指定する。実行後は、「load all」で再読込する。
#                   例: time 0.6
#
#   mess            messageを表示する
#   message         例: mess "topoSetEditor\n 選択partsが表示されます"
#
#   addMess         messageを追加表示する
#   addMessage      例: addMess 追加表示
#
#   stl             stlFileの場所を指定する。実行後は「load stls」で再読込する
#   stlDir          例: stl ./model1
#
#   edge            edge表示を制御する
#                   例: edge <option>  option: show or hide
#
#   outline         outline表示を制御する
#                   例: outline <option>  option: show or hide
#
#   face            face表裏の表示を制御する
#                   例: face <option>  option: both, front or back
#
#
#   21/08/05    新規作成
#      09/04    showItemSize:minLocationの座標の表示を追加。
#      09/15    setsのvtk作成をmeshViewerCreateSetsVtk.pyで作成。
#               foamToVtkコマンドを使っていたが、OF-v2006では出力結果
#               が異なる為、共用できない。
#      09/23    commandLoad:delayを設定して実行する様に修正。
#               大規模メッシュの場合、loadに時間がかかる為。
#

import logFileCreater
logFileCreater.log()

print("importing vtk...")
import vtk
from vtk.util import colors
print("  imported " + vtk.vtkVersion.GetVTKSourceVersion())

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gdk, GLib
import os, sys
import glob
import threading
import time

import GtkVtkRenderWindowInteractor as gtkVtk
import pyTreeFoam
import meshViewerCreateSetsVtk as createSetsVtk


#-------------------
#  windowApp class
#-------------------
class windowApp:

    def __init__(self, caseDir, timeFolder, region, inputFile):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path + "meshViewerDialog.glade")
        self.mainWindow = self.builder.get_object("window1")
        self.mainWindow.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        #GUIのobjectを取得
        self.maskEvent = True
        self.setGtkObject()
        #変数の設定
        self.caseDir = self.getAbsPath(caseDir)
        self.timeFolder = timeFolder
        self.region = region
        self.stlDir = caseDir + "/model"
        self.inputFile = inputFile
        self.closeFlag = 0
        #vtk関連
        self.vtkObj = gtkVtk.GtkVtkRenderWindowInteractor(self)
        self.vtkSize = (300, 250)
        #actorの辞書
        self.patchActorDict = {}
        self.outlineActorDict = {}
        self.cellZoneActorDict = {}
        self.faceZoneActorDict = {}
        self.pointZoneActorDict = {}
        self.cellSetActorDict = {}
        self.faceSetActorDict = {}
        self.pointSetActorDict = {}
        self.stlActorDict = {}
        self.currItemActors = []
        self.axes = ""
        self.renderer = ""

    #
    #  main
    #--------
    def main(self):
        """ GUIの表示"""
        #vtkAreaをセット
        self.setVtkArea()
        #actorを取得
        # self.iniGetAllActors()
        # self.getPatchesZonesActors()
        # self.getSetsActors()
        # self.getStlActors()
        #GUIを表示
        self.mainWindow.show_all()
        #vtkを表示
        self.setIniVtkModel()              #vtkを表示
        self.currItemActors = []
        self.showActors(self.currItemActors)
        self.resetCamera()
        #コマンド取得thread起動
        self.runThreadGetCommand()
        self.maskEvent = False
        #loop
        Gtk.main()

    #
    #  close
    #---------
    def close(self, *args):
        """ 閉じる"""
        self.closeFlag = 1
        Gtk.main_quit()

    #--------- event handler ----------
    def onChangeShowEdge(self, event):      #edge表示
        if self.maskEvent == False:
            self.changeShowEdge()
    def onChangeShowOutline(self, event):   #outline表示
        if self.maskEvent == False:
            self.changeShowOutline()
    def onResetCamera(self, event):         #resetCamera
        self.resetCamera()
    def onSetViewX(self, event):            #X軸視
        self.setViewX()
    def onSetViewY(self, event):            #Y軸視
        self.setViewY()
    def onSetViewZ(self, event):            #Z軸視
        self.setViewZ()
    def onRightToLeft(self, event):         #左右前後入れ替え
        self.rightToLeft()
    def onRolling90(self, event):           #回転90°
        self.rolling90()
    def onReloadMesh(self, event):          #再読込
        self.reloadMesh()
    def onChangeShowBoth(self, event):      #face両面表示
        self.changeShowBoth()
    def onChangeShowFront(self, event):     #face表面表示
        self.changeShowFront()
    def onChangeShowBack(self, event):      #face裏面表示
        self.changeShowBack()

    #
    #  setGtkObject
    #----------------
    def setGtkObject(self):
        """ glade内のobjectを取得"""
        #vtk関連
        self.box_vtk = self.builder.get_object("box_vtk")
        self.check_showEdge = self.builder.get_object("check_showEdge")
        self.check_showOutline = self.builder.get_object("check_showOutline")
        self.button_resetCamera = self.builder.get_object("button_resetCamera")
        self.button_viewX = self.builder.get_object("button_viewX")
        self.button_viewY = self.builder.get_object("button_viewY")
        self.button_viewZ = self.builder.get_object("button_viewZ")
        self.button_rightToLeft = self.builder.get_object("button_rightToLeft")
        self.button_rolling90 = self.builder.get_object("button_rolling90")
        self.button_reload = self.builder.get_object("button_reload")
        self.radio_showBoth = self.builder.get_object("radio_showBoth")
        self.radio_showFront = self.builder.get_object("radio_showFront")
        self.radio_showBack = self.builder.get_object("radio_showBack")
        self.label_message = self.builder.get_object("label_message")
        self.label_status = self.builder.get_object("label_status")
        self.label_itemSize = self.builder.get_object("label_itemSize")
        #初期設定
        self.check_showEdge.set_active(True)
        self.check_showOutline.set_active(True)
        #tooltip
        self.check_showEdge.props.tooltip_text = _("表示モデルのedgeを表示する")
        self.check_showOutline.props.tooltip_text = _("モデル全体の概形状を表示する")
        self.button_resetCamera.props.tooltip_text = _("画面にフィットさせる（reset camera）")
        self.button_viewX.props.tooltip_text = _("x軸視（y-z 面）を表示")
        self.button_viewY.props.tooltip_text = _("y軸視（z-x 面）を表示")
        self.button_viewZ.props.tooltip_text = _("z軸視（x-y 面）を表示")
        self.button_rightToLeft.props.tooltip_text = _("視点を反転(左右前後反転)")
        self.button_rolling90.props.tooltip_text = _("時計回りに90度回転")
        self.button_reload.props.tooltip_text = _("vtkDataを再読込する")
        self.radio_showBoth.props.tooltip_text = _("faceの表裏両面を表示する")
        self.radio_showFront.props.tooltip_text = _("faceの表面のみ表示する\nfaceの方向が確認できる")
        self.radio_showBack.props.tooltip_text = _("faceの裏面のみ表示する\n隠れた内部のstlが確認できる")

    #
    #  setVtkArea
    #-------------
    def setVtkArea(self):
        """ vtkAreaを設定する"""
        vtkObj = self.vtkObj
        vtkObj.set_size_request(*self.vtkSize)
        self.box_vtk.pack_start(vtkObj, True, True, 0)

    #
    #  addCornerAxis
    #----------------
    def addCornerAxis(self):
        """ XYZ軸（compas）をrendererに追加する"""
        interactor = self.renWin.GetInteractor()
        axesActor = vtk.vtkAxesActor()
        self.axes = vtk.vtkOrientationMarkerWidget()
        self.axes.SetOrientationMarker(axesActor)
        self.axes.SetInteractor(interactor)
        self.axes.EnabledOn()
        self.axes.InteractiveOn()

    #
    #  setIniVtkModel
    #---------------
    def setIniVtkModel(self):
        """ vtk画面の初期化"""
        self.renderer = vtk.vtkRenderer()
        #backgroundColorを設定
        self.renderer.GradientBackgroundOn()
        self.renderer.SetBackground(0.85, 0.85, 0.85)
        self.renderer.SetBackground2(0.4, 0.55, 0.75)
        #renderWindowをセット
        self.renWin = self.vtkObj.GetRenderWindow()
        self.renWin.NewInstance()
        #rendererを追加
        self.renWin.AddRenderer(self.renderer)
        #axisを追加
        self.addCornerAxis()
        #vtk表示
        self.vtkObj.GlobalWarningDisplayOff()   #windowsのエラー対策
        self.vtkObj.Initialize()
        self.vtkObj.Start()

    #
    #  showActors
    #-------------
    def showActors(self, itemActors):
        """ actorを表示する"""
        #現在のactorを削除する
        delActors = self.renderer.GetActors()
        for actor in delActors:
            self.renderer.RemoveActor(actor)
        #outlineをチェック
        actors = []
        if self.check_showOutline.get_active() == True:
            outlineActors = list(self.outlineActorDict.values())
            actors += outlineActors
        #指定のactorを取得し、propertyを設定
        for item, actor in itemActors:
            if item == "outline":
                pass
            else:
                actor = self.setPropertyOfActor(actor, item)
                actors.append(actor)
        #指定のactorを表示させる
        for actor in actors:
            self.renderer.AddActor(actor)
        #itemSizeを表示
        bounds = self.getBoundsActors()
        self.showItemSize(bounds)
        self.vtkObj.Initialize()

    #
    #  showItemSize
    def showItemSize(self, bounds):
        """ 表示itemのサイズを表示する"""
        xyz = []; minxyz = []
        if len(bounds) > 0:
            #lengthを取得
            x = bounds[1]-bounds[0]
            y = bounds[3]-bounds[2]
            z = bounds[5]-bounds[4]
            xstr = pyTreeFoam.getValue3digit_shrink(x)
            ystr = pyTreeFoam.getValue3digit_shrink(y)
            zstr = pyTreeFoam.getValue3digit_shrink(z)
            xyz = [xstr, ystr, zstr]
            #minLocationを取得
            minx = bounds[0]
            miny = bounds[2]
            minz = bounds[4]
            minxyz = [pyTreeFoam.getValue3digit_shrink(minx),
                      pyTreeFoam.getValue3digit_shrink(miny),
                      pyTreeFoam.getValue3digit_shrink(minz)]
        cont = _("表示itemのサイズ XYZ: ") + " ".join(xyz)
        cont += _("  (minLoc: ") + " ".join(minxyz) + " )"
        self.label_itemSize.set_text(cont)


    #--------------------
    #  eventの処理
    #--------------------
    #
    #  changeShowEdge
    #-----------------
    def changeShowEdge(self):
        """ showEdgeを変更した時"""
        self.showActors(self.currItemActors)

    #
    #  changeShowOutline
    #--------------------
    def changeShowOutline(self):
        """ shouOutlineを変更した時"""
        self.showActors(self.currItemActors)

    #
    #  resetCamera
    #---------------
    def resetCamera(self):
        """ 画面に合わせてVTKを表示させる"""
        bounds = self.getCameraBounds()
        self.renderer.ResetCamera(*bounds)
        self.vtkObj.Initialize()

    #
    #  setViewX
    #-----------
    def setViewX(self):
        """ X軸方向から見る"""
        #camera位置を算出
        bounds = self.getBoundsActors()
        Xcenter = (bounds[0] + bounds[1]) / 2.0
        Ycenter = (bounds[2] + bounds[3]) / 2.0
        Zcenter = (bounds[4] + bounds[5]) / 2.0
        maxLeng = max(bounds[1]-bounds[0], bounds[3]-bounds[2], bounds[5]-bounds[4])
        cameraPos = (Xcenter+maxLeng*2.0, Ycenter, Zcenter)
        #X軸視に設定
        camera = self.renderer.GetActiveCamera()
        #  camera位置を設定
        camera.SetPosition(*cameraPos)
        #  cameraの縦軸の単位vectorを設定
        camera.SetViewUp(0, 0, 1)
        #  表示
        bounds = self.getCameraBounds()
        self.renderer.ResetCamera(*bounds)
        self.vtkObj.Initialize()

    #
    #  setViewY
    #-----------
    def setViewY(self):
        """ Y軸視（Z-X面）"""
        #camera位置を算出
        bounds = self.getBoundsActors()
        Xcenter = (bounds[0] + bounds[1]) / 2.0
        Ycenter = (bounds[2] + bounds[3]) / 2.0
        Zcenter = (bounds[4] + bounds[5]) / 2.0
        maxLeng = max(bounds[1]-bounds[0], bounds[3]-bounds[2], bounds[5]-bounds[4])
        cameraPos = (Xcenter, Ycenter+maxLeng*2.0, Zcenter)
        #Y軸視に設定
        camera = self.renderer.GetActiveCamera()
        #  camera位置を設定
        camera.SetPosition(*cameraPos)
        #  cameraの縦軸の単位vectorを設定
        camera.SetViewUp(1, 0, 0)
        #  表示
        bounds = self.getCameraBounds()
        self.renderer.ResetCamera(*bounds)
        self.vtkObj.Initialize()

    #
    #  setViewZ
    #-----------
    def setViewZ(self):
        """ Z軸視（X-Y面）"""
        #camera位置を算出
        bounds = self.getBoundsActors()
        Xcenter = (bounds[0] + bounds[1]) / 2.0
        Ycenter = (bounds[2] + bounds[3]) / 2.0
        Zcenter = (bounds[4] + bounds[5]) / 2.0
        maxLeng = max(bounds[1]-bounds[0], bounds[3]-bounds[2], bounds[5]-bounds[4])
        cameraPos = (Xcenter, Ycenter, Zcenter+maxLeng*2.0)
        #Z軸視に設定
        camera = self.renderer.GetActiveCamera()
        #  camera位置を設定
        camera.SetPosition(*cameraPos)
        #  cameraの縦軸の単位vectorを設定
        camera.SetViewUp(0, 1, 0)
        #  表示
        bounds = self.getCameraBounds()
        self.renderer.ResetCamera(*bounds)
        self.vtkObj.Initialize()

    #
    #  rightToLeft
    #--------------
    def rightToLeft(self):
        """ camera位置を反転（左右反転）させる"""
        bounds = self.getBoundsActors()
        Xcenter = (bounds[0] + bounds[1]) / 2.0
        Ycenter = (bounds[2] + bounds[3]) / 2.0
        Zcenter = (bounds[4] + bounds[5]) / 2.0
        center = (Xcenter, Ycenter, Zcenter)
        camera = self.renderer.GetActiveCamera()
        pos = camera.GetPosition()
        relPos = (pos[0]-center[0], pos[1]-center[1], pos[2]-center[2])
        newRelPos = list(map(lambda x: -x, relPos))
        newPos = (newRelPos[0]+center[0], newRelPos[1]+center[1], newRelPos[2]+center[2])
        camera.SetPosition(*newPos)
        #  表示
        bounds = self.getCameraBounds()
        self.renderer.ResetCamera(*bounds)
        self.vtkObj.Initialize()

    #
    #  rolling90
    #-------------
    def rolling90(self):
        """ 視点を右回転（90°）させる。"""
        camera = self.renderer.GetActiveCamera()
        camera.Roll(-90.0)
        #  表示
        bounds = self.getCameraBounds()
        self.renderer.ResetCamera(*bounds)
        self.vtkObj.Initialize()

    #
    #  reloadMesh
    #-------------
    def reloadMesh(self):
        """ meshDataをloadする"""
        self.remakeSetsVtk()
        print("loading patches, zones, sets, stl...")
        self.commandLoad("")

    #
    #  remakeSetsVtk
    def remakeSetsVtk(self):
        """ setsのvtkを作り直す"""
        caseDir = self.caseDir
        region = self.region
        timeFolder = self.timeFolder
        #vtkを無条件で作り直す。
        createSetsVtk.createAllSetsVtkFiles(caseDir, timeFolder, region, checkFlag=False)

    #
    #  changeShowBoth
    #-----------------
    def changeShowBoth(self):
        """ face両面を表示"""
        self.showActors(self.currItemActors)

    #
    #  changeShowFront
    #-------------------
    def changeShowFront(self):
        """ face表面を表示"""
        self.showActors(self.currItemActors)

    #
    #  changeShowBack
    #-----------------
    def changeShowBack(self):
        """ face裏面を表示"""
        self.showActors(self.currItemActors)

    #  getCameraBounds
    def getCameraBounds(self):
        """ 表示させるactorのboundsを取得する。
        マージンを含めて取得する。"""
        bounds = self.getBoundsActors()
        if len(bounds) == 0:
            return ()
        #マージンを確保
        xmin = bounds[0]
        xmax = bounds[1]
        ymin = bounds[2]
        ymax = bounds[3]
        zmin = bounds[4]
        zmax = bounds[5]
        a = 0.1
        xmin = xmin - (xmax-xmin) * a
        xmax = xmax + (xmax-xmin) * a
        ymin = ymin - (ymax-ymin) * a
        ymax = ymax + (ymax-ymin) * a
        zmin = zmin - (zmax-xmin) * a
        zmax = zmax + (zmax-xmin) * a
        return (xmin, xmax, ymin, ymax, zmin, zmax)

    #  getBoundsActors
    def getBoundsActors(self):
        """ 全actorのboundsを取得して返す"""
        actors = self.renderer.GetActors()
        bounds = []
        for actor in actors:
            #if actor in self.actorDict.values():
            bound = actor.GetBounds()
            bounds.append(bound)
        if len(bounds) == 0:
            return ()
        Xmin = min(map(lambda x: x[0], bounds))
        Xmax = max(map(lambda x: x[1], bounds))
        Ymin = min(map(lambda x: x[2], bounds))
        Ymax = max(map(lambda x: x[3], bounds))
        Zmin = min(map(lambda x: x[4], bounds))
        Zmax = max(map(lambda x: x[5], bounds))
        return (Xmin, Xmax, Ymin, Ymax, Zmin, Zmax)


    #--------------------------
    #  showActorのcommand関連
    #--------------------------
    #
    #  runThreadGetCommand
    #----------------------
    def runThreadGetCommand(self):
        """ コマンド取得するthreadを起動する"""
        th = threading.Thread(target=self.getCommandLine)
        th.start()

    #
    #  getCommandLine
    def getCommandLine(self):
        """ コマンドを取得する"""
        if self.inputFile == "":
            #key入力
            while True:
                try:
                    line = input()
                except:
                    break
                words = line.split()
                if len(words) >= 1:
                    if words[0] == "close" or words[0] == "exit" or words[0] == "quit":
                        if self.closeFlag == 0:
                            self.close()
                        break
                    else:
                        GLib.timeout_add(10, self.runCommand, line)
                if self.closeFlag == 1:
                    break
                time.sleep(0.2)            
            return
        else:
            #fileから取得する
            pid = os.getpid()
            path = os.getenv("TreeFoamUserPath")
            fileName = path + "/temp/meshViewer_" + str(pid)
            f = open(fileName, encoding="utf-8")
            line = ""
            while True:
                line = f.readline()
                if line != "":
                    if line[:len("close")] == "close":
                        #「close」コマンドの場合、戻る
                        f.close()
                        self.close()
                        break
                    else:
                        GLib.timeout_add(10, self.runCommand, line)
                        #time.sleep(1.0)
                #GUIを閉じた時、戻る
                if self.closeFlag == 1:
                    f.close()
                    break
            return

    #
    #  runCommand
    def runCommand(self, line):
        """ コマンドを解釈し、実行する。"""

        #  skipUntilNextWord
        def skipUntilNextWord(line):
            lineOp = pyTreeFoam.strOp(line)
            keyword, p = lineOp.getKeyword(0)
            pm = lineOp.skipSP(p)
            if pm >= len(line):
                keyword += line[-1]
            #「"」「'」を削除
            newLine = line[pm:].replace("'", "")
            newLine = newLine.replace('"', '')
            newLine = newLine.replace("\\n", "\n")
            return newLine

        words = line.split()
        if len(words) <= 0:
           return
        command = words[0]
        argLine = skipUntilNextWord(line)
        if command[0] == "#":
            pass
        elif command == "show" or command == "showActor":
            self.writeStatus(line)
            self.commandShowActor(argLine)
        elif command == "addShow" or command == "addShowActor":
            self.writeStatus(line)
            self.commandAddShowActor(argLine)
        elif command == "load" or command == "reload":
            self.writeStatus(line+"...")
            self.commandLoad(argLine)
        elif command == "case" or command == "caseDir":
            self.writeStatus(line)
            self.commandCaseDir(argLine)
        elif command == "region":
            self.writeStatus(line)
            self.commandRegion(argLine)
        elif command == "message" or command == "mess":
            self.writeStatus(line)
            self.commandMessage(argLine)
        elif command == "addMessage" or command == "addMess":
            self.writeStatus(line)
            self.commandAddMessage(argLine)
        elif command == "stl" or command == "stlDir":
            self.writeStatus(line)
            self.commandStlDir(argLine)
        elif command == "time":
            self.writeStatus(line)
            self.commandTime(argLine)
        elif command == "outline":
            self.writeStatus(line)
            self.commandOutline(argLine)
        elif command == "edge":
            self.writeStatus(line)
            self.commandEdge(argLine)
        elif command == "face":
            self.writeStatus(line)
            self.commandFace(argLine)
        elif command == "resetCamera":
            self.writeStatus(line)
            self.commandResetCamera(argLine)
        else:
            print(_("error: コマンド「") + command + _("」が間違っています。"))

    #
    #  commandShowActor
    def commandShowActor(self, line):
        """ vtkのactorを表示する"""
        args = line.split()
        actors = self.getCommandArgsActors(args)
        self.currItemActors = actors[:]
        self.showActors(self.currItemActors)

    #
    #  commandAddShowActor
    def commandAddShowActor(self, line):
        """ vtkのactorを追加して表示する"""
        args = line.split()
        currActors = self.currItemActors[:]
        actors = self.getCommandArgsActors(args)
        allActors = currActors + actors
        self.currItemActors = allActors[:]
        self.showActors(self.currItemActors)

    #
    #  commandLoad
    def commandLoad(self, line):
        """ meshDataをloadする"""
        #0.1sのdeleyを掛けて直ぐに戻る。
        #commandLoad実行前の0.1sの間にGUIを完成させる為
        GLib.timeout_add(100, self.commandLoad_sub, line)

    #
    #  commandLoad_sub
    def commandLoad_sub(self, line):
        """ meshDataをloadする"""
        args = line.split()
        if len(args) == 0:
            self.iniGetAllActors()
            self.getPatchesZonesActors()
            self.getSetsActors()
            self.getStlActors()
        else:
            for arg in args:
                if arg == "all":
                    self.iniGetAllActors()
                    self.getPatchesZonesActors()
                    self.getSetsActors()
                    self.getStlActors()
                elif arg == "patches" or arg == "zones" or arg == "patchesZones":
                    self.iniGetPatchesZonesActors()
                    self.getPatchesZonesActors()
                elif arg == "sets":
                    self.iniGetSetsActors()
                    self.getSetsActors()
                elif arg == "stls" or arg == "stl":
                    self.getStlActors()
                else:
                    print(_("error: 引数「") + arg + _("」が間違っています。"))
        self.currItemActors = []
        self.showActors(self.currItemActors)
        self.writeStatus(line)

    #
    #  commandCaseDir
    def commandCaseDir(self, argLine):
        """ caseDirを変更する"""
        self.caseDir = self.getAbsPath(argLine)

    #  getAbsPath
    def getAbsPath(self, path):
        """ 絶対pathに変換する"""
        path = os.path.normpath(path)
        path = os.path.expanduser(path)
        path = os.path.abspath(path)
        return path

    #
    #  commandRegion
    def commandRegion(self, argLine):
        """ regionを変更する"""
        region = argLine
        self.region = region

    #
    #  commandMessage
    def commandMessage(self, argLine):
        """ messageを取得し、表示する"""
        if argLine[-1] == "\n":
            argLine = argLine[:-1]
        self.label_message.set_text(argLine)

    #
    #  commandAddMessage
    def commandAddMessage(self, argLine):
        """ 追加するmessageを取得し、追加表示する"""
        text = self.label_message.get_text()
        if argLine[-1] == "\n":
            argLine = argLine[:-1]
        newText = text + "\n" + argLine
        self.label_message.set_text(newText)

    #
    #  commandStlDir
    def commandStlDir(self, argLine):
        """ stlDirを設定する"""
        self.stlDir = argLine
        if self.stlDir[0] != "/":
            self.stlDir = self.caseDir + "/" + self.stlDir
        self.stlDir = os.path.normpath(self.stlDir)

    #
    #  commandTime
    def commandTime(self, argLine):
        """ timeFolderを設定する。"""
        try:
            a = float(argLine)
            self.timeFolder = argLine
        except:
            pass

    #
    #  commandOutline
    def commandOutline(self, argLine):
        """ outlineの表示を制御"""
        if argLine == "show":
            self.check_showOutline.set_active(True)
        elif argLine == "hide":
            self.check_showOutline.set_active(False)

    #
    #  commandEdge
    def commandEdge(self, argLine):
        """ edgeの表示を制御"""
        if argLine == "show":
            self.check_showEdge.set_active(True)
        elif argLine == "hide":
            self.check_showEdge.set_active(False)

    #
    #  commandFace
    def commandFace(self, argLine):
        """ faceの表示を制御"""
        if argLine == "both":
            self.radio_showBoth.set_active(True)
        elif argLine == "front":
            self.radio_showFront.set_active(True)
        elif argLine == "back":
            self.radio_showBack.set_active(True)

    #
    #  commandResetCamera
    def commandResetCamera(self, argLine):
        """ resetCameraを実行する。"""
        self.resetCamera()

    #  writeStatus
    def writeStatus(self, line):
        """ status行に表示させる"""
        print(line)
        line = line.split("\n")[0]
        if len(line) > 97:
            line = line[:97] + "..."
        self.label_status.set_text(line)

    #  getCommandArgsActors
    def getCommandArgsActors(self, args):
        """ commandのargs内のactorを翻訳して取得する。"""
        actorDict = {
            "patch":    self.patchActorDict,
            "cellZone": self.cellZoneActorDict,
            "faceZone": self.faceZoneActorDict,
            "pointZone":self.pointZoneActorDict,
            "cellSet":  self.cellSetActorDict,
            "faceSet":  self.faceSetActorDict,
            "pointSet": self.pointSetActorDict,
            "stl":      self.stlActorDict
            }
        actors = []
        for arg in args:
            words = arg.split(":")
            item = words[0]
            if item in actorDict.keys():
                getDict = actorDict[item]
                name = words[1]
                if item == "stl":
                    if name[:-4] != ".stl":
                        name += ".stl"
                if name in getDict.keys():
                    actor = getDict[name]
                    itemActor = [item, actor]
                    actors.append(itemActor)
        return actors

    #
    #  setPropertyOfActor
    def setPropertyOfActor(self, actor, item):
        """ actorのpropertyを設定"""
        if item == "outline":
            return
        #edge表示
        prop = actor.GetProperty()
        if self.check_showEdge.get_active() == True:
            prop.EdgeVisibilityOn()
        else:
            prop.EdgeVisibilityOff()
        #表裏表示の設定
        if self.radio_showBoth.get_active() == True:
            prop.BackfaceCullingOff()
            prop.FrontfaceCullingOff()
        elif self.radio_showFront.get_active() == True:
            prop.BackfaceCullingOn()
            prop.FrontfaceCullingOff()
        elif self.radio_showBack.get_active() == True:
            prop.BackfaceCullingOff()
            prop.FrontfaceCullingOn()
        return actor

    #----------------------
    #  actor取得関連
    #----------------------
    #
    #  iniGetAllActors
    #---------------
    def iniGetAllActors(self):
        """ 全actor取得関連の定義"""
        self.iniGetPatchesZonesActors()
        self.iniGetSetsActors()

    #
    #  iniGetPatchesZonesActors
    def iniGetPatchesZonesActors(self):
        """ patchActor, zoneActor取得関連の定義"""
        #patch, zonesの初期化定義
        contFile = self.caseDir + "/system/controlDict"
        self.readerOF = vtk.vtkOpenFOAMReader()
        self.readerOF.SetFileName(contFile)
        #  patch読込
        #self.readerOF.Update()
        self.readerOF.UpdateTimeStep(float(self.timeFolder))
        self.readerOF.EnableAllPatchArrays()
        self.readerOF.SetReadZones(True)
        #  zones読込
        self.readerOF.Update()

    #
    #  iniGetSetsActors
    def iniGetSetsActors(self):
        """ setActor取得関連の定義"""
        caseDir = self.caseDir
        region = self.region
        timeFolder = self.timeFolder
        createSetsVtk.createAllSetsVtkFiles(caseDir, timeFolder, region, checkFlag=True)
        folderDir = createSetsVtk.getMeshDir(caseDir, timeFolder, region)
        vtkFiles = [[], [], []]
        vtkDir = folderDir + "/VTK"
        files = glob.glob(vtkDir + "/*")
        for fileName in files:
            name = os.path.basename(fileName)
            setType = name.split("_")[0]
            if setType == "cellSet":
                vtkFiles[0].append(fileName)
            elif setType == "faceSet":
                vtkFiles[1].append(fileName)
            elif setType == "pointSet":
                vtkFiles[2].append(fileName)
        self.vtkFiles = vtkFiles
        return

    #  checkItemVtkFile
    def checkItemVtkFile(self, vtkFilePtn):
        """ vtkFileをチェックし、存在すれば削除する。"""
        delFiles = glob.glob(vtkFilePtn)
        if len(delFiles) > 0:
            for delFile in delFiles:
                os.remove(delFile)

    #
    #  getPatchesZonesActors
    #------------------------
    def getPatchesZonesActors(self):
        """ patchとzonesのactorを辞書で取得"""
        #blockDataを取得
        blockData = self.getBlockDataForReading()
        if blockData != None:
            #actorを取得
            patchActorDicts, zoneActorDicts = self.getRegionData(blockData)
            (self.patchActorDict, self.outlineActorDict) = patchActorDicts
            (self.cellZoneActorDict, self.faceZoneActorDict, self.pointZoneActorDict) = zoneActorDicts
        else:
            self.patchActorDict = {}
            self.outlineActorDict = {}
            self.cellZoneActorDict = {}
            self.faceZoneActorDict = {}
            self.pointZoneActorDict = {}
        return

    #
    #  getBlockDataForReading
    def getBlockDataForReading(self):
        """ 最初のblockを取得（region有無で変わるため）"""
        output = self.readerOF.GetOutput()
        blockData = None
        if self.region == ".":
            #region0を取得
            metaData = output.GetMetaData(0)
            blockName = self.getBlockName(metaData)
            if blockName == "internalMesh":
                blockData = output
            else:
                blockData = output.GetBlock(0)
        else:
            #regionのblockDataを取得
            nBlocks = output.GetNumberOfBlocks()
            for i in range(nBlocks):
                metaData = output.GetMetaData(i)
                blockName = self.getBlockName(metaData)
                if blockName == self.region:
                    blockData = output.GetBlock(i)
                    break
        return blockData

    #
    #  getBlockName
    def getBlockName(self, metaData):
        """ metaDataからそのblockのblockNameを取得する"""
        lines = str(metaData).split("\n")
        blockName = ""
        for line in lines:
            words = line.split()
            if words[0] == "NAME:":
                blockName = words[1]
                break
        return blockName

    #
    #  getRegionData
    def getRegionData(self, regionBlock):
        """ regionのblockDataからactorDictを取得する"""
        nBlocks = regionBlock.GetNumberOfBlocks()
        #blockDataからpatchDataを取得
        patchActorDict = {}; edgeActorDict = {}
        for i in range(nBlocks):
            metaData = regionBlock.GetMetaData(i)
            blockName = self.getBlockName(metaData)
            if blockName == "Patches":
                patchBlock = regionBlock.GetBlock(i)
                patchNamesIds = self.getPatchNamesIds(patchBlock)
                patchActorDict, edgeActorDict = self.createPatchActors(regionBlock, patchNamesIds)
                break
        #blockDataからzoneDataを取得
        zoneActorDicts = [{}, {}, {}]
        for i in range(nBlocks):
            metaData = regionBlock.GetMetaData(i)
            blockName = self.getBlockName(metaData)
            if blockName == "Zones":
                zoneBlock = regionBlock.GetBlock(i)
                zoneActorDicts = self.createAllZoneActors(zoneBlock)
                break
        patchActorDicts = [patchActorDict, edgeActorDict]
        return patchActorDicts, zoneActorDicts

    #  getPatchNamesIds
    def getPatchNamesIds(self, patchBlock):
        """ blockDataからpatchNameとpatchIdを取得する"""
        patchIds = []
        patchNames = []
        index = 3
        nBlocks = patchBlock.GetNumberOfBlocks()
        for i in range(nBlocks):
            metaData = patchBlock.GetMetaData(i)
            patchName = self.getBlockName(metaData)
            patchIds.append(index)
            patchNames.append(patchName)
            index += 1
        return patchNames, patchIds

    #  createPatchActors
    def createPatchActors(self, regionBlock, patchNamesIds):
        """ patchActor, outlineActor(featureEdge)を作成する"""
        (patchNames, patchIds) = patchNamesIds
        patchActorDict = {}
        outlineActorDict = {}
        for i in range(len(patchIds)):
            index = patchIds[i]
            extract_block = vtk.vtkExtractBlock()
            extract_block.SetInputData(regionBlock)
            extract_block.AddIndex(index)
            extract_block.Update()
            #filter
            geom_filter = vtk.vtkGeometryFilter()
            geom_filter.SetInputConnection(extract_block.GetOutputPort())
            #featureEdge
            feature_edge = vtk.vtkFeatureEdges()
            feature_edge.SetInputConnection(geom_filter.GetOutputPort())
            #mapper
            mapper = vtk.vtkCompositePolyDataMapper()
            mapper.SetInputConnection(geom_filter.GetOutputPort())
            mapper.ScalarVisibilityOff()
            edgeMapper = vtk.vtkCompositePolyDataMapper()
            edgeMapper.SetInputConnection(feature_edge.GetOutputPort())
            edgeMapper.ScalarVisibilityOff()
            #actor
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            prop = actor.GetProperty()
            prop.SetColor(colors.green)
            name = patchNames[i]
            patchActorDict[name] = actor
            edgeActor = vtk.vtkActor()
            edgeActor.SetMapper(edgeMapper)
            outlineActorDict[name] = edgeActor
        return patchActorDict, outlineActorDict

    #  createAllZoneActors
    def createAllZoneActors(self, zoneBlock):
        """ 全zoneのactorを取得"""
        cellZoneActorDict = {}
        faceZoneActorDict = {}
        pointZoneActorDict = {}
        nBlocks = zoneBlock.GetNumberOfBlocks()
        for i in range(nBlocks):
            metaData = zoneBlock.GetMetaData(i)
            zoneName = self.getBlockName(metaData)
            if zoneName == "cellZones":
                block = zoneBlock.GetBlock(i)
                cellZoneNamesIds = self.getZoneNamesIds(block)
                cellZoneActorDict = self.createCellActors(block, cellZoneNamesIds)
            elif zoneName == "faceZones":
                block = zoneBlock.GetBlock(i)
                faceZoneNamesIds = self.getZoneNamesIds(block)
                faceZoneActorDict = self.createZoneActors(block, faceZoneNamesIds)
            elif zoneName == "pointZones":
                block = zoneBlock.GetBlock(i)
                pointZoneNamesIds = self.getZoneNamesIds(block)
                pointZoneActorDict = self.createZoneActors(block, pointZoneNamesIds)
        actorDicts = (cellZoneActorDict, faceZoneActorDict, pointZoneActorDict)
        return actorDicts

    #  createCellActors
    def createCellActors(self, block, cellZoneNamesIds):
        """ 表面のみのcellZoneActorを作成する。"""
        zoneNames, zoneIds = cellZoneNamesIds
        cellActorDict = {}
        for i in range(len(zoneIds)):
            index = zoneIds[i]
            extract_block = vtk.vtkExtractBlock()
            extract_block.SetInputData(block)
            extract_block.AddIndex(index)
            extract_block.Update()
            #filter
            #geom_filter = vtk.vtkGeometryFilter()
            geom_filter = vtk.vtkDataSetSurfaceFilter()
            geom_filter.SetInputConnection(extract_block.GetOutputPort())
            #mapper
            mapper = vtk.vtkCompositePolyDataMapper()
            mapper.SetInputConnection(geom_filter.GetOutputPort())
            mapper.SetScalarModeToUseCellFieldData()
            mapper.ScalarVisibilityOff()
            #actor
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            prop = actor.GetProperty()
            prop.SetColor(colors.blue)
            name = zoneNames[i]
            cellActorDict[name] = actor
        return cellActorDict

    #
    #  createZoneActors
    def createZoneActors(self, block, zoneNamesIds):
        """ faceとpointのzoneActorを作成する"""
        zoneNames, zoneIds = zoneNamesIds
        zoneActorDict = {}
        for i in range(len(zoneIds)):
            index = zoneIds[i]
            extract_block = vtk.vtkExtractBlock()
            extract_block.SetInputData(block)
            extract_block.AddIndex(index)
            extract_block.Update()
            #filter
            geom_filter = vtk.vtkGeometryFilter()
            geom_filter.SetInputConnection(extract_block.GetOutputPort())
            #mapper
            mapper = vtk.vtkCompositePolyDataMapper()
            mapper.SetInputConnection(geom_filter.GetOutputPort())
            mapper.ScalarVisibilityOff()
            #actor
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            name = zoneNames[i]
            prop = actor.GetProperty()
            prop.SetColor(colors.blue)
            #prop.BackfaceCullingOn()
            #prop.SetOpacity(0.5)
            zoneActorDict[name] = actor
        return zoneActorDict

    #  getZoneNamesIds
    def getZoneNamesIds(self, block):
        """ blockDataからzoneNameとzoneIdを取得する"""
        zoneNames = []
        zoneIds = []
        index = 1
        nBlocks = block.GetNumberOfBlocks()
        for i in range(nBlocks):
            metaData = block.GetMetaData(i)
            zoneName = self.getBlockName(metaData)
            zoneNames.append(zoneName)
            zoneIds.append(index)
            index += 1
        return zoneNames, zoneIds

    #
    #  getSetsActors
    #----------------
    def getSetsActors(self):
        """ setsのactorを辞書で作成する。"""
        self.cellSetActorDict = self.createCellVtkActors(self.vtkFiles[0])
        self.faceSetActorDict = self.createFaceVtkActors(self.vtkFiles[1])
        self.pointSetActorDict = self.createPointVtkActors(self.vtkFiles[2])

    #
    #  getStlActors
    #---------------
    def getStlActors(self):
        """ stlのactorの辞書を作成する。"""
        actorDict = {}
        stlFiles = glob.glob(self.stlDir + "/*.stl")
        for stlFile in stlFiles:
            reader = vtk.vtkSTLReader()
            reader.SetFileName(stlFile)
            reader.Update()
            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInputConnection(reader.GetOutputPort())
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            name = os.path.basename(stlFile)
            actorDict[name] = actor
        self.stlActorDict = actorDict

    #  createCellVtkActors
    def createCellVtkActors(self, fileNames):
        """ cellSetのactorを作成"""
        actorDict = {}
        for fileName in fileNames:
            reader = vtk.vtkUnstructuredGridReader()
            reader.SetFileName(fileName)
            reader.Update()
            output = reader.GetOutput()
            mapper = vtk.vtkDataSetMapper()
            mapper.SetInputData(output)
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            prop = actor.GetProperty()
            prop.SetColor(colors.blue_light)
            name = self.getSetNameFromFile(fileName)
            actorDict[name] = actor
        return actorDict

    #  getSetNameFromFile
    def getSetNameFromFile(self, fileName):
        """ fileNameからsetNameを取得"""
        name = os.path.basename(fileName)
        #name = "_".join(name.split("_")[:-1])
        name = "_".join(name.split("_")[1:])
        name = name[:-4]
        return name

    #  createFaceVtkActors
    def createFaceVtkActors(self, fileNames):
        """ faceSetのactorを作成"""
        actorDict = {}
        for fileName in fileNames:
            #reader = vtk.vtkPolyDataReader()
            reader = vtk.vtkUnstructuredGridReader()
            reader.SetFileName(fileName)
            reader.Update()
            output = reader.GetOutput()
            mapper = vtk.vtkDataSetMapper()
            mapper.SetInputData(output)
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            prop = actor.GetProperty()
            prop.SetColor(colors.blue_light)
            name = self.getSetNameFromFile(fileName)
            actorDict[name] = actor
        return actorDict

    #  createPointVtkActors
    def createPointVtkActors(self, fileNames):
        """ pointSetのactorを作成"""
        actorDict = {}
        for fileName in fileNames:
            reader = vtk.vtkPolyDataReader()
            #reader = vtk.vtkStructuredPointsReader()
            reader.SetFileName(fileName)
            reader.Update()
            output = reader.GetOutput()
            mapper = vtk.vtkPointGaussianMapper()
            mapper.SetInputData(output)
            mapper.EmissiveOff()
            mapper.SetScaleFactor(0.0)
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            prop = actor.GetProperty()
            prop.SetColor(colors.blue_light)
            name = self.getSetNameFromFile(fileName)
            actorDict[name] = actor
        return actorDict

#
#  getArguments
#---------------
def getArguments():
    """ 引数を取得する。"""
    caseDir = os.getcwd()
    timeFolder = "0"
    region = "."
    inputFile = ""
    i = 1
    while i < len(sys.argv):
        arg = sys.argv[i]
        if arg == "-case":
            i += 1
            caseDir = sys.argv[i]
        elif arg == "-time":
            i += 1
            timeFolder = sys.argv[i]
        elif arg == "-region":
            i += 1
            region = sys.argv[i]
        elif arg == "-input":
            i += 1
            inputFile = sys.argv[i]
        i += 1
    return caseDir, timeFolder, region, inputFile


if __name__ == "__main__":
    import gettext
    import locale
    localeDir = os.getenv("TreeFoamPath") + "/data/locale"
    locale.bindtextdomain("treefoam", localeDir)
    gettext.install("treefoam", localeDir)
    #gettext.install("app")

    #引数を取得
    caseDir, timeFolder, region, inputFile = getArguments()
    #GUIを表示
    winApp = windowApp(caseDir, timeFolder, region, inputFile)
    winApp.main()
